@@ -30,9 +30,10 @@
 #define DEH_VANILLA_NUMSFX 107
 
 void DEH_ParseCommandLine(void);
-int DEH_LoadFile(char *filename);
+int DEH_LoadFile(const char *filename);
+void DEH_AutoLoadPatches(const char *path);
 int DEH_LoadLump(int lumpnum, boolean allow_long, boolean allow_error);
-int DEH_LoadLumpByName(char *name, boolean allow_long, boolean allow_error);
+int DEH_LoadLumpByName(const char *name, boolean allow_long, boolean allow_error);
 
 boolean DEH_ParseAssignment(char *line, char **variable_name, char **value);
 
