@@ -77,8 +77,6 @@ typedef struct
 
 #define MAXANIMS                32
 
-extern anim_t	anims[MAXANIMS];
-extern anim_t*	lastanim;
 
 //
 // P_InitPicAnims
@@ -135,8 +133,8 @@ anim_t*		lastanim;
 //
 #define MAXLINEANIMS            64
 
-extern  short	numlinespecials;
-extern  line_t*	linespeciallist[MAXLINEANIMS];
+short numlinespecials;
+line_t *linespeciallist[MAXLINEANIMS];
 
 
 
@@ -149,7 +147,7 @@ void P_InitPicAnims (void)
     lastanim = anims;
     for (i=0 ; animdefs[i].istexture != -1 ; i++)
     {
-        char *startname, *endname;
+        const char *startname, *endname;
 
         startname = DEH_String(animdefs[i].startname);
         endname = DEH_String(animdefs[i].endname);
@@ -509,24 +507,36 @@ P_CrossSpecialLine
 
     line = &lines[linenum];
     
-    //	Triggers that other things can activate
+    if (gameversion <= exe_doom_1_2)
+    {
+        if (line->special > 98 && line->special != 104)
+        {
+            return;
+        }
+    }
+    else
+    {
+        //	Triggers that other things can activate
+        if (!thing->player)
+        {
+            // Things that should NOT trigger specials...
+            switch(thing->type)
+            {
+                case MT_ROCKET:
+                case MT_PLASMA:
+                case MT_BFG:
+                case MT_TROOPSHOT:
+                case MT_HEADSHOT:
+                case MT_BRUISERSHOT:
+                    return;
+
+                default: break;
+            }
+        }
+    }
+
     if (!thing->player)
     {
-	// Things that should NOT trigger specials...
-	switch(thing->type)
-	{
-	  case MT_ROCKET:
-	  case MT_PLASMA:
-	  case MT_BFG:
-	  case MT_TROOPSHOT:
-	  case MT_HEADSHOT:
-	  case MT_BRUISERSHOT:
-	    return;
-	    break;
-	    
-	  default: break;
-	}
-		
 	ok = 0;
 	switch(line->special)
 	{
@@ -1182,8 +1192,6 @@ static void DonutOverrun(fixed_t *s3_floorheight, short *s3_floorpic,
     static int tmp_s3_floorheight;
     static int tmp_s3_floorpic;
 
-    extern int numflats;
-
     if (first)
     {
         int p;
@@ -1366,9 +1374,20 @@ int EV_DoDonut(line_t*	line)
 // After the map has been loaded, scan for specials
 //  that spawn thinkers
 //
-short		numlinespecials;
-line_t*		linespeciallist[MAXLINEANIMS];
 
+static unsigned int NumScrollers()
+{
+    unsigned int i, scrollers = 0;
+
+    for (i = 0; i < numlines; i++)
+    {
+        if (48 == lines[i].special)
+        {
+            scrollers++;
+        }
+    }
+    return scrollers;
+}
 
 // Parses command line parameters.
 void P_SpawnSpecials (void)
@@ -1447,9 +1466,13 @@ void P_SpawnSpecials (void)
 	    P_SpawnDoorRaiseIn5Mins (sector, i);
 	    break;
 	    
-	  case 17:
-	    P_SpawnFireFlicker(sector);
-	    break;
+        case 17:
+            // first introduced in official v1.4 beta
+            if (gameversion > exe_doom_1_2)
+            {
+                P_SpawnFireFlicker(sector);
+            }
+            break;
 	}
     }
 
@@ -1463,8 +1486,8 @@ void P_SpawnSpecials (void)
 	  case 48:
             if (numlinespecials >= MAXLINEANIMS)
             {
-                I_Error("Too many scrolling wall linedefs! "
-                        "(Vanilla limit is 64)");
+                I_Error("Too many scrolling wall linedefs (%d)! "
+                        "(Vanilla limit is 64)", NumScrollers());
             }
 	    // EFFECT FIRSTCOL SCROLL+
 	    linespeciallist[numlinespecials] = &lines[i];
