@@ -21,7 +21,7 @@
 
 boolean I_SetVideoModeHR(void);
 void I_UnsetVideoModeHR(void);
-void I_SetWindowTitleHR(char *title);
+void I_SetWindowTitleHR(const char *title);
 void I_ClearScreenHR(void);
 void I_SlamBlockHR(int x, int y, int w, int h, const byte *src);
 void I_SlamHR(const byte *buffer);
