@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "i_system.h"
 #include "m_misc.h"
 #include "sha1.h"
 #include "w_checksum.h"
@@ -44,7 +45,7 @@ static int GetFileNumber(wad_file_t *handle)
     // Not found in list.  This is a new file we haven't seen yet.
     // Allocate another slot for this file.
 
-    open_wadfiles = realloc(open_wadfiles,
+    open_wadfiles = I_Realloc(open_wadfiles,
                             sizeof(wad_file_t *) * (num_open_wadfiles + 1));
     open_wadfiles[num_open_wadfiles] = handle;
 
